'  2006 Artyom Krivokrisenko
' Web Reflection, Development Department

Option Strict On

Imports System.Xml.XPath

Public Class MainForm

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        BindParent(LoadData("data.xml"))
    End Sub

    Sub BindParent(ByVal data As List(Of MyItem))
        Parent.DataSource = Nothing
        Parent.DataSource = data
        Parent.DisplayMember = "Name"
    End Sub

    Sub BindChild(ByVal data As List(Of MyItem))
        Child.DataSource = Nothing
        Child.DataSource = data
        Child.DisplayMember = "Name"
    End Sub

    Function LoadData(ByVal fileName As String) As List(Of MyItem)
        Dim Result As New List(Of MyItem)

        Dim Doc As New XPathDocument(fileName)
        Dim Nav As XPathNavigator = Doc.CreateNavigator

        For Each SubNav As XPathNavigator In Nav.Select("/items/item")
            Result.Add(LoadNode(SubNav))
        Next
        Return Result
    End Function

    Function LoadNode(ByVal Nav As XPathNavigator) As MyItem
        Dim Result As New MyItem
        Result.Name = Nav.GetAttribute("name", "")

        For Each SubNav As XPathNavigator In Nav.Select("item")
            Result.ChildItems.Add(LoadNode(SubNav))
        Next

        Return Result
    End Function

    Private Sub Parent_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Parent.SelectedIndexChanged
        If Parent.SelectedItem IsNot Nothing Then
            BindChild(CType(Parent.SelectedItem, MyItem).ChildItems)
        Else
            Child.DataSource = Nothing
        End If
    End Sub

    Private Sub Up_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Up.Click
        Dim CurrentItem As MyItem = CType(Parent.SelectedItem, MyItem)
        If CurrentItem IsNot Nothing Then
            Dim ParentList As List(Of MyItem) = CType(Parent.DataSource, List(Of MyItem))
            Dim ItemPosition As Integer = ParentList.IndexOf(CurrentItem)
            If ItemPosition > 0 Then
                ParentList(ItemPosition) = ParentList(ItemPosition - 1)
                ParentList(ItemPosition - 1) = CurrentItem
                BindParent(ParentList)
                Parent.SelectedItem = CurrentItem
            End If
        End If
    End Sub

    Private Sub Down_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Down.Click
        Dim CurrentItem As MyItem = CType(Parent.SelectedItem, MyItem)
        If CurrentItem IsNot Nothing Then
            Dim ParentList As List(Of MyItem) = CType(Parent.DataSource, List(Of MyItem))
            Dim ItemPosition As Integer = ParentList.IndexOf(CurrentItem)
            If ItemPosition < ParentList.Count - 1 Then
                ParentList(ItemPosition) = ParentList(ItemPosition + 1)
                ParentList(ItemPosition + 1) = CurrentItem
                BindParent(ParentList)
                Parent.SelectedItem = CurrentItem
            End If
        End If
    End Sub

    Private Sub Remove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Remove.Click
        Dim CurrentItem As MyItem = CType(Parent.SelectedItem, MyItem)
        If CurrentItem IsNot Nothing Then
            Dim ParentList As List(Of MyItem) = CType(Parent.DataSource, List(Of MyItem))
            ParentList.Remove(CurrentItem)
            BindParent(ParentList)
        End If
    End Sub
End Class